@@ -82,7 +82,7 @@ class HandleWalletFunctions():
                 else:
                     WalletDict['address'] = addy_seed[1].split(":")[-1].lstrip().rstrip()
                 WalletDict['seed'] = lines[-1].lstrip().rstrip().replace('\n', '')
-                remove(WALLETINFO)
+                #remove(WALLETINFO)
                 return WalletDict
     
         else:
@@ -95,19 +95,24 @@ class HandleWalletFunctions():
         CONFIG = MeileGuiConfig.read_configuration(MeileGuiConfig, MeileGuiConfig.CONFFILE)
         PASSWORD = CONFIG['wallet'].get('password', '')
     
-        ofile =  open(SUBSCRIBEINFO, "wb")    
+        ofile =  open(SUBSCRIBEINFO, "wb")
+            
+        if not KEYNAME:
+            return (False, 1337)
         
         SCMD = "%s tx subscription subscribe-to-node --yes --keyring-backend file --keyring-dir %s --gas-prices 0.1udvpn --chain-id sentinelhub-2 --node https://rpc.mathnodes.com:443 --from '%s' '%s' %s"  % (sentinelcli, KEYRINGDIR, KEYNAME, NODE, DEPOSIT)    
-     
-        child = pexpect.spawn(SCMD)
-        child.logfile = ofile
-        
-        child.expect(".*")
-        child.sendline(PASSWORD)
-        child.expect(pexpect.EOF)
-        
-        ofile.flush()
-        ofile.close()
+        try:
+            child = pexpect.spawn(SCMD)
+            child.logfile = ofile
+            
+            child.expect(".*")
+            child.sendline(PASSWORD)
+            child.expect(pexpect.EOF)
+            
+            ofile.flush()
+            ofile.close()
+        except pexpect.exceptions.TIMEOUT:
+            return (False, 1415)
         
         return self.ParseSubscribe(self)
         
